Agent/TCP set rfc2988_ false

# Get the simulation parameters (-param1 value1 -param2 value2...)
for {set i 0} {$i < $argc} {incr i} {
    global opt
    set arg [lindex $argv $i]
    if {[string range $arg 0 0] != "-"} continue
    set name [string range $arg 1 end]
    set opt($name) [lindex $argv [expr $i+1]]
}

#####

load /home/jlakkako/ns-allinone-2.35/dei80211mr-1.1.4/src/.libs/libdei80211mr.so

Node/MobileNode instproc getIfq { param0} {
    $self instvar ifq_    
    return $ifq_($param0) 
}

Node/MobileNode instproc getPhy { param0} {
    $self instvar netif_    
    return $netif_($param0) 
}

set per [new PER]
$per loadPERTable80211gTrivellato
# Pn = kTB. k is Boltzmann's constant, 1.38*10^-23 J/K, T is the temperature in Kelvin (room temperature, 290 K), and B is the system bandwidth [Hz].
$per set noise_ 9.75e-12 ; # -80 dBm

# HP iPAQ 500 Series Voice Messenger
# Power consumption: transmit mode: 1.33 W, receive mode: 1.235 W
Phy/WirelessPhy set CSThresh_ 1e-10 ; # -70 dBm
Phy/WirelessPhy set Pt_ 0.0178      ; # 12.5 dBm
Phy/WirelessPhy set freq_ 2437e6
Phy/WirelessPhy set L_ 1.0

Mac/802_11/Multirate set useShortPreamble_ true
Mac/802_11/Multirate set gSyncInterval_ 0.00001
Mac/802_11/Multirate set CWMin_         16
Mac/802_11/Multirate set CWMax_         1024
Mac/802_11/Multirate set VerboseCounters_ 1
Mac/802_11/Multirate set RTSThreshold_ 0
Mac/802_11/Multirate set ShortRetryLimit_ 8
Mac/802_11/Multirate set LongRetryLimit_  5
Mac/802_11/Multirate set SlotTime_ 0.000009
Mac/802_11/Multirate set SIFS_ 0.000016

Mac/802_11 set useShortPreamble_ true
Mac/802_11 set gSyncInterval_ 0.00001
Mac/802_11 set CWMin_         16
Mac/802_11 set CWMax_         1024
Mac/802_11 set VerboseCounters_ 1
Mac/802_11 set RTSThreshold_ 0
Mac/802_11 set ShortRetryLimit_ 8
Mac/802_11 set LongRetryLimit_  5
Mac/802_11 set SlotTime_ 0.000009
Mac/802_11 set SIFS_ 0.000016

PeerStatsDB set VerboseCounters_ 1

#####

# ======================================================================
# Define options
# ======================================================================
set val(chan)           Channel/WirelessChannel/PowerAware
set val(prop)           Propagation/FreeSpace/PowerAware
set val(netif)          Phy/WirelessPhy/PowerAware
set val(mac)            Mac/802_11/Multirate
set val(ifq)            Queue/DropTail/PriQueue    ;# interface queue type
set val(ll)             LL                         ;# link layer type
set val(ant)            Antenna/OmniAntenna        ;# antenna model
set val(ifqlen)         100000                     ;# max packet in ifq
set val(nn)             $opt(nn)                   ;# number of mobilenodes
set val(rp)             AODV                       ;# routing protocol
set val(energyModel)    EnergyModel 

set peerstats [new PeerStatsDB/Static]
$peerstats numpeers $val(nn)

global defaultRNG
$defaultRNG seed $opt(seed)

# Create a simulator object
set ns_ [new Simulator]

# Open a trace file
set tracefd [open /dev/null w]
$ns_ trace-all $tracefd

# Define a 'finish' procedure
proc finish {} {
    global ns_ tracefd node_ val
    for {set i 0} {$i < $val(nn) } {incr i} {
	puts "Energy at node $i at t = [$ns_ now]: [format "%.4f" [$node_($i) energy]]"
    }
    $ns_ flush-trace
    close $tracefd
    exit 0
}

# Set up topography object
set topo [new Topography]
$topo load_flatgrid 4400.0 3400.0 ; # NOTE: Depends on the mobility trace file!

# Create God
set god_ [create-god $val(nn)]

# Node configuration
set chan [new $val(chan)]
$ns_ node-config -adhocRouting $val(rp) \
    -llType $val(ll) \
    -macType $val(mac) \
    -ifqType $val(ifq) \
    -ifqLen $val(ifqlen) \
    -antType $val(ant) \
    -propType $val(prop) \
    -phyType $val(netif) \
    -channel $chan \
    -topoInstance $topo \
    -energyModel $val(energyModel) \
    -idlePower 0.83 \
    -rxPower 1.0 \
    -txPower 1.4 \
    -sleepPower 0.001 \
    -transitionPower 0.2 \
    -transitionTime 0.005 \
    -initialEnergy 100000 \
    -agentTrace ON \
    -routerTrace ON \
    -macTrace ON \
    -phyTrace ON \
    -movementTrace ON

for {set i 0} {$i < $val(nn) } {incr i} {
    set node_($i) [$ns_ node] 
    
    set mac_($i) [$node_($i) getMac 0]
    set ifq_($i) [$node_($i) getIfq 0]
    set phy_($i) [$node_($i) getPhy 0]
    
    $mac_($i) nodes $val(nn)
    $mac_($i) basicMode_ Mode6Mb
    $mac_($i) dataMode_ Mode54Mb
    $mac_($i) per $per
    $mac_($i) PeerStatsDB $peerstats
    set pp_($i) [new PowerProfile]
    $mac_($i) powerProfile $pp_($i) 
    $phy_($i) powerProfile $pp_($i) 

    set ra_($i) [new RateAdapter/SNR]
    $ra_($i) set pktsize_ 1500
    $ra_($i) set maxper_ 0.1778
    $ra_($i) attach2mac $mac_($i)
    $ra_($i) use80211g
    $ra_($i) setmodeatindex 0

    $node_($i) random-motion 0
    $node_($i) start
}

if {($opt(nn) == 116)&&($opt(simtime) == 3600)} {
    source ../alltraces/centre_MovementNs2Report.txt
} else {
    puts "Wrong set of parameters for any mobility tracefile!"
}

Agent/TCP instproc done {ttl} {
    global ns_
    $self instvar node_ fid_
    puts "Source $fid_ (at node: [$node_ id]) received last ack at t = [$ns_ now]. Hop count = [expr 31-$ttl]."
}

source ./trafficgen_tcp.tcl

for {set i 0} {$i < $val(nn) } {incr i} {
    $ns_ at $opt(simtime) "$node_($i) reset";
    puts "Energy at node $i at t = [$ns_ now]: [format "%.4f" [$node_($i) energy]]"
}

$ns_ at  [expr $opt(simtime)+0.1] "finish"

# Run the simulation
$ns_ run